/**
 -------------------------------------------------------------------------------------------------------------------------------
This file provides boilerplate templates for interfacing with the GUI Composer framework.
For further information, see the available options under the Help main menu in the Designer.
-------------------------------------------------------------------------------------------------------------------------------
**/

import { GcUtils } from './components/@ti/gc-core-assets/lib/GcUtils';
import { GcConsole } from './components/@ti/gc-core-assets/lib/GcConsole';
import { bindingRegistry } from './components/@ti/gc-core-databind/lib/CoreDatabind';
import { GcWidget } from './components/@ti/gc-widget-base/lib/GcWidget';
import { ActionRegistry } from './components/@ti/gc-widget-menu/lib/ActionRegistry';
import { codecRegistry } from './components/@ti/gc-target-configuration/lib/TargetConfiguration';

const console = new GcConsole('myapp'); // creates a console instance with name 'myapp'
GcConsole.setLevel('myapp', 4);         // enable console output for myapp console instance
console.info('index.js is loaded...');

/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for databinding

Add custom computed value databindings here, using the following method:

syntax: bindingRegistry.bind(targetBinding, sourceBinding, [getter], [setter]);
    param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
    param sourceBinding - single binding string or expression, or array of binding strings for multi-way binding.
    param getter - (optional) - custom getter function for computing the targetBinding value(s) based on sourceBinding value(s).
    param setter - (optional) - custom setter function for computing the sourceBinding value(s) based on targetBinding value(s).

  (async () => {
      await bindingRegistry.waitForModelReady('widget');        // widget model, a built-in model
      await bindingRegistry.waitForModelReady('targetModelId'); // target model, gc-model-program, gc-model-streaming, etc...
      bindingRegistry.bind('widget.inputWidgetId.value', 'model.targetModelId.targetVariable');
      bindingRegistry.bind('widget.labelWidgetId.label', 'widget.inputWidgetId.value');
   })();
-------------------------------------------------------------------------------------------------------------------------------
**/

// Example code:
//
// (async () => {
//     /* Wait for widget and target models to be ready */
//     await bindingRegistry.waitForModelReady('widget');
//     await bindingRegistry.waitForModelReady('targetModelId');
//
//     /* A simple computed values based on simple expression */
//     bindingRegistry.bind('widget.id.propertyName', "targetModelId.targetVariable == 1 ? 'binding is one' : 'binding is not one'");
//
//     /* A custom two-way binding with custom getter and setter functions */
//     /* (setter is optional and getter only indicates one-way binding)   */
//     bindingRegistry.bind('widget.id.propertyName', 'targetModelId.targetVariable',
//         value => { return value*5/9 + 32; }, /* getter */
//         value => { return (value-32)*9/5; }  /* setter */
//     );
//
//     /* 1 to n bindings */
//     bindingRegistry.bind('widget.date.value', {
//         /* Dependant bindings needed in order to compute the date, in name/value pairs */
//             weekday: 'widget.dayOfWeek.selectedLabel',
//             day: 'widget.dayOfMonth.value',
//             month: 'widget.month.selectedLabel',
//             year: 'widget.year.value'
//         },
//         /* Getter for date computation */
//         function(values) {
//             /* compute and return the string value to bind to the widget with id 'date' */
//             return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
//         }
//     );
// })();


/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for working with webcomponents in the application
-------------------------------------------------------------------------------------------------------------------------------
**/

const init = () => {
    // Add menubar product-name-clicked event listener
    GcWidget.querySelector('gc-widget-menubar').then(menubar => {
        menubar.addEventListener('product-name-clicked', () => window.open('https://dev.ti.com/gc', 'gc'));
    });
    
    /*GcWidget.querySelector('#page_collateral').then(page_collateral => {
        
        page_collateral.addEventListener('click', () => {
        
            const usb2any = codecRegistry.getInstance('usb2any');
        
            codecRegistry.whenConfigurationReady('usb2any').then( () => {
        
                const Cmd_SPI_Control = 8;
                const Cmd_SPI_WriteAndRead = 9;
    
                let Byte1 = 0;
                let Byte2 = 0;
    
                usb2any.readResponse(usb2any.sendCommandPacket(9, [2,Byte1,Byte2]));
    
                });
            });
        });*/
        
    /*let packet = [
        PACKET_IDENTIFIER, 0, payload.length, COMMAND_PACKET, 0, this.txPacketSeqNum++, 0, cmd
    ];
    
    packet = packet.concat(payload);
    packet[PACKET_PEC] = calculateCRC(packet, PACKET_PAYLOAD_LEN);*/
    

    GcWidget.querySelector('#main_tab_container').then(tab => {
        tab.addEventListener('index-changed', (currentIndex) => {
            if (currentIndex.detail.value == 5) //tabs are 0 based index, thus last tab is 4
            {
                const usb2any = codecRegistry.getInstance('usb2any');
        
                codecRegistry.whenConfigurationReady('usb2any').then( () => {
        
                    const Cmd_SPI_Control = 8;
                    const Cmd_SPI_WriteAndRead = 9;
    
                    let Byte1 = 0x00;
                    let Byte2 = 0x00;
    
                    usb2any.readResponse(usb2any.sendCommandPacket(9, [2,Byte1,Byte2]));
    
                });
            }
        });
    });

        
        GcWidget.querySelector('#enable_daisy_chain_button').then(daisyChainButton => {
            daisyChainButton.addEventListener('click', () => {
        
            const usb2any = codecRegistry.getInstance('usb2any');
            codecRegistry.whenConfigurationReady('usb2any').then( () => {
            
                const Cmd_SPI_Control = 8;
                const Cmd_SPI_WriteAndRead = 9; // make it only write 

                let Byte1 = 0x25;
                let Byte2 = 0x00;

                usb2any.readResponse(usb2any.sendCommandPacket(9, [2,Byte1,Byte2]));
    
            });
        });
    });

    


    for(let i = 0; i < 10; i++)
    {
        for(let j = 0; j < 8; j++)
        {
        eval(`GcWidget.querySelector('#daisy_chain_device${i+1}_switch${j+1}_checkbox').then(device${i+1}Switch${j+1}Checkbox => {
            device${i+1}Switch${j+1}Checkbox.addEventListener('checked-changed', (event) => {
                SwitchInDaisyChain()
                });
            });`);            
        }
        
    }
    
};

function SwitchInDaisyChain ()
{
    //Create an array that contains all enabled 
    const packet = [];
    for(let i = 0; i < daisy_chain_select.selectedValue; i++)
    {
        let currByte = 0;
        for(let j = 0; j < 8; j++)
        {
            currByte += eval(`daisy_chain_device${daisy_chain_select.selectedValue-i}_switch${j+1}_checkbox.checked`)*(2**(j));
        }
        packet[i+1]=currByte;
    }
    packet[0] = daisy_chain_select.selectedValue;
    const usb2any = codecRegistry.getInstance('usb2any');
                    codecRegistry.whenConfigurationReady('usb2any').then( () =>{
                        const Cmd_SPI_Control = 8;
                        const Cmd_SPI_WriteAndRead = 9; 

                        usb2any.readResponse(usb2any.sendCommandPacket(Cmd_SPI_WriteAndRead, packet));
                        
                    }); 

    return 0;
    
}

document.readyState === 'complete' ? init() : document.addEventListener('DOMContentLoaded', init);


/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for registering menu and toolbar action callback
-------------------------------------------------------------------------------------------------------------------------------
**/

ActionRegistry.registerAction('cmd_exit', {
    run() { GcUtils.isNW ? require('nw.gui').Window.get().close() : window.close(); }
});